<?php
/**
 * An interface for package delivery boys... ;-)
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
interface Deliverable extends FrameworkInterface {
	/**
	 * "Enqueues" raw content into this delivery class by reading the raw content
	 * from given helper's template instance and pushing it on the 'undeclared'
	 * stack.
	 *
	 * @param	$helperInstance		An instance of a HubHelper class
	 * @return	void
	 */
	function enqueueRawDataFromTemplate (HubHelper $helperInstance);

	/**
	 * Checks whether a package has been enqueued for delivery.
	 *
	 * @return	$isEnqueued		Whether a package is enqueued
	 */
	function isPackageEnqueued ();

	/**
	 * Checks whether a package has been declared
	 *
	 * @return	$isDeclared		Whether a package is declared
	 */
	function isPackageDeclared ();

	/**
	 * Checks whether a package should be sent out
	 *
	 * @return	$isWaitingDelivery	Whether a package is waiting for delivery
	 */
	function isPackageWaitingForDelivery ();

	/**
	 * Checks whether encoded (raw) data is pending
	 *
	 * @return	$isPending	Whether encoded data is pending
	 */
	function isEncodedDataPending ();

	/**
	 * Delivers an enqueued package to the stated destination. If a non-session
	 * id is provided, recipient resolver is being asked (and instanced once).
	 * This allows that a single package is being delivered to multiple targets
	 * without enqueueing it for every target. If no target is provided or it
	 * can't be determined a NoTargetException is being thrown.
	 *
	 * @return	void
	 * @throws	NoTargetException	If no target can't be determined
	 */
	function declareEnqueuedPackage ();

	/**
	 * Processes the next declared package. Only one package per time will be
	 * processed because this may take time and slows down the whole delivery
	 * infrastructure.
	 *
	 * @return	void
	 */
	function processDeclaredPackage ();

	/**
	 * Sends waiting packages out for delivery
	 *
	 * @return	void
	 */
	function sendWaitingPackage ();

	/**
	 * Sends pending encoded (raw) data
	 *
	 * @return	void
	 */
	function sendEncodedData ();

	/**
	 * Clears all stacks
	 *
	 * @return	void
	 */
	function clearAllStacks ();
}

// [EOF]
?>
