<?php
/**
 * A factory class for network packages
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class PackageTagsFactory extends ObjectFactory {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Returns a singleton network package instance. If an instance is found in
	 * the registry it will be returned, else a new instance is created and
	 * stored in the same registry entry.
	 *
	 * @return	$packageInstance	A network package instance
	 */
	public static final function createPackageTagsInstance () {
		// Do we have an instance in the registry?
		if (Registry::getRegistry()->instanceExists('package_tags')) {
			// Then use this instance
			$packageInstance = Registry::getRegistry()->getInstance('package_tags');
		} else {
			// Now prepare the tags instance
			$packageInstance = self::createObjectByConfiguredName('package_tags_class');

			// Set the instance in registry for further use
			Registry::getRegistry()->addInstance('package_tags', $packageInstance);
		}

		// Return the instance
		return $packageInstance;
	}
}

// [EOF]
?>
