<?php
/**
 * A hub-mcrypt miner class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2014 Miner Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class HubCoinMiner extends BaseHubMiner implements MinerHelper, Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set version number
		$this->setVersion('0.0.0');
	}

	/**
	 * Creates an instance of this hub-miner class
	 *
	 * @return	$minerInstance	An instance of this hub-miner class
	 */
	public final static function createHubCoinMiner () {
		// Get a new instance
		$minerInstance = new HubCoinMiner();

		// Return the instance
		return $minerInstance;
	}

	/**
	 * This method fills the in-buffer with (a) test unit(s) which are mainly
	 * used for development of the crunching part. They must be enabled in
	 * configuration, or else your miner runs out of WUs and waits for more
	 * to show up.
	 *
	 * In this method we already know that the in-buffer is going depleted so
	 * no need to double-check it here.
	 *
	 * @return	void
	 */
	protected function fillInBufferQueueWithTestUnits () {
		// Are test units enabled?
		if ($this->getConfigInstance()->getConfigEntry('miner_test_units_enabled') == 'N') {
			// They are disabled, so skip any further steps
			return;
		} elseif ($this->getStateInstance()->isMinerStateVirgin()) {
			// No virgin miners please, because they usually have no test units ready for crunching
			return;
		}

		// Get a test-unit generator instance
		$generatorInstance = ObjectFactory::createObjectByConfiguredName('miner_test_unit_generator_class');

		// We don't need an iterator here because we just need to generate some test units
		for ($idx = 0; $idx < $this->getConfigInstance()->getConfigEntry('miner_max_text_unit_amount'); $idx++) {
			// Get a test unit from it
			$unitInstance = $generatorInstance->generateNextUnitInstance();

			// ... and finally queue it to the in-buffer queue
			$this->queueUnitInstanceToInBuffer($unitInstance);
		} // END - for
	}

	/**
	 * This method fills the in-buffer with (real) WUs which will be crunched
	 * and the result be sent back to the key producer instance.
	 *
	 * @return	void
	 */
	protected function fillInBufferQueueWithWorkUnits () {
		// This miner's state must not be one of these: 'virgin'
		if ($this->getStateInstance()->isMinerStateVirgin()) {
			// We can silently skip here, until the generation is finished
			return;
		} // END - if

		// @TODO Implement this method
		$this->partialStub('Please implement this method.');
	}

	/**
	 * Method to "bootstrap" the miner. This step does also apply provided
	 * command-line arguments stored in the request instance. No buffer queue
	 * will be initialized here, we only do "general" things here.
	 *
	 * @return	void
	 * @todo	Implement this method
	 */
	public function doBootstrapping () {
		$this->partialStub('Please implement this method.');
	}

	/**
	 * Outputs the console teaser. This should only be executed on startup or
	 * full restarts. This method generates some space around the teaser.
	 *
	 * @return	void
	 */
	public function outputConsoleTeaser () {
		// Output all lines
		self::createDebugInstance(__CLASS__)->debugOutput(' ');
		self::createDebugInstance(__CLASS__)->debugOutput('Hubcoin miner v' . $this->getVersion() . ' is starting ...');
		self::createDebugInstance(__CLASS__)->debugOutput('Copyright (c) 2014 Miner Developer Team');
		self::createDebugInstance(__CLASS__)->debugOutput(' ');
		self::createDebugInstance(__CLASS__)->debugOutput('This program comes with ABSOLUTELY NO WARRANTY; for details see docs/COPYING.');
		self::createDebugInstance(__CLASS__)->debugOutput('This is free software, and you are welcome to redistribute it under certain');
		self::createDebugInstance(__CLASS__)->debugOutput('conditions; see docs/COPYING for details.');
		self::createDebugInstance(__CLASS__)->debugOutput(' ');
	}

	/**
	 * Add some miner-specific filters
	 *
	 * @param	$controllerInstance	An object of a Controller instance
	 * @param	$responseInstance	An object of a Responseable instance
	 * @return	void
	 * @todo	0% done
	 */
	public function addExtraFilters (Controller $controllerInstance, Responseable $responseInstance) {
		// Add some filters here
		$this->partialStub('Please add some miner-specific filters, if required.');
	}
}

// [EOF]
?>
