<?php
/**
 * A Upper recipient
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class UpperRecipient extends BaseRecipient implements Recipient {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$recipientInstance		An instance of a Recipient class
	 */
	public final static function createUpperRecipient () {
		// Get new instance
		$recipientInstance = new UpperRecipient();

		// Return the prepared instance
		return $recipientInstance;
	}

	/**
	 * Tries to resolve given recipient into session ids or Universal Node Locator
	 * depending on implementation (hint: Template Method Pattern)
	 *
	 * @param	$recipient		Recipient to resolve (e.g. could be a virtual recipient or direct session id)
	 * @param	$listInstance	An instance of a Listable class
	 * @param	$packageData	Valid package data array
	 * @return	$resolved		Resolved recipient or VOID if only the set list has been filled
	 * @throws	FrameworkException	Could throw different exceptions depending on implementation
	 */
	public function resolveRecipient ($recipient, Listable $listInstance, array $packageData) {
		// Make sure the recipient is valid
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('UPPER-RECIPIENT: recipient=' . $recipient);
		assert($recipient == NetworkPackage::NETWORK_TARGET_UPPER);

		// Get all bootstrap nodes
		foreach (explode(BaseHubSystem::BOOTSTRAP_NODES_SEPARATOR, $this->getConfigInstance()->getConfigEntry('hub_bootstrap_nodes')) as $unlData) {
			// Is maximum reached?
			if ($listInstance->count() == $this->getConfigInstance()->getConfigEntry('package_recipient_max_count')) {
				// Debug message
				/* DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('RECIPIENT-RESOLVER[' . __METHOD__ . ':' . __LINE__ . ']: Going to abort at maximum of ' . $this->getConfigInstance()->getConfigEntry('package_recipient_max_count') . ' recipients!');

				// Then stop adding more
				break;
			} // END - if

			// Debug message
			/* DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('RECIPIENT-RESOLVER[' . __METHOD__ . ':' . __LINE__ . ']: Adding node ' . print_r($unlData, TRUE) . ' as recipient.');

			// Add the entry
			$listInstance->addEntry('unl', $unlData);
		} // END - foreach
	}
}

// [EOF]
?>
