<?php

error_reporting(E_ALL | E_STRICT);
set_time_limit(0);

require('udp-inc.php');

$port = 60825;
$host = '0.0.0.0';

$leaving = array('/leave', '/logout', '/bye', '/quit');
$shutdown = array('/shutdown', '/halt');
$masters = array('127.0.0.1');

// create a streaming socket, of type TCP/IP
$main_sock = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);

// set the option to reuse the port
socket_set_option($main_sock, SOL_SOCKET, SO_REUSEADDR, 1);

// "bind" the socket to the address to "localhost", on port $port
// so this means that all connections on this port are now our resposibility to send/recv data, disconnect, etc..
socket_bind($main_sock, $host, $port);

// start listen for connections
socket_listen($main_sock);

// create a list of all the clients that will be connected to us..
// add the listening socket to this list
$clients = array($main_sock);

out(__FILE__, __LINE__, '['.date('m/d/Y:H:i:s', time())."]:Server listens on {$host}:{$port}");

while (count($clients) > 0) {
	// create a copy, so $clients doesn't get modified by socket_select()
	$read = $clients;

	// get a list of all the clients that have data to be read from
	// if there are no clients with data, go to next iteration
	$left = @socket_select($read, $write = null, $except = null, 0, 150);
	if ($left < 1) {
		continue;
	}

	// check if there is a client trying to connect
	if (in_array($main_sock, $read)) {
		// accept the client, and add him to the $clients array
		$new_sock = socket_accept($main_sock);
		$clients[] = $new_sock;

		// send the client a welcome message
		socket_write($new_sock, "No noobs, but I'll make an exception. :)\n".
		"There are ".(count($clients) - 1)." client(s) connected to the server.\n");

		socket_getpeername($new_sock, $ip);
		out(__FILE__, __LINE__, '['.date('m/d/Y:H:i:s', time())."]:New client connected: {$ip}");

		// Notify all chatter
		if (count($clients) > 2) {
			foreach ($clients as $send_sock) {
				if ($send_sock != $main_sock && $send_sock != $new_sock) {
					socket_write($send_sock, "Server: Chatter has joined from {$ip}. There are now ".(count($clients) - 1)." clients.\n");
				}
			}
		}

		// remove the listening socket from the clients-with-data array
		$key = array_search($main_sock, $read);
		unset($read[$key]);
	}

	// loop through all the clients that have data to read from
	foreach ($read as $read_sock) {
		// Get client data
		socket_getpeername($read_sock, $ip);

		// read until newline or 1024 bytes
		// socket_read while show errors when the client is disconnected, so silence the error messages
		$data = @socket_read($read_sock, 1024, PHP_NORMAL_READ);

		// check if the client is disconnected
		if (($data === FALSE) || (in_array(strtolower(trim($data)), $leaving))) {

			// remove client for $clients array
			$key = array_search($read_sock, $clients);
			unset($clients[$key]);
			out(__FILE__, __LINE__, '['.date('m/d/Y:H:i:s', time())."]:Client from {$ip} disconnected. Left: ".(count($clients) - 1)."");

			// Notify all chatter
			if (count($clients) > 1) {
				foreach ($clients as $send_sock) {
					if ($send_sock != $main_sock) {
						socket_write($send_sock, "Server: Chatter from {$ip} has logged out. ".(count($clients) - 1)." client(s) left.\n");
					}
				}
			}

			// continue to the next client to read from, if any
			socket_write($read_sock, "Server: Good bye.\n");
			socket_shutdown($read_sock, 2);
			socket_close($read_sock);
			continue;
		} elseif (in_array(trim($data), $shutdown)) {
			// Is he allowed to shutdown?
			if (!in_array($ip, $masters)) {
				out(__FILE__, __LINE__, '['.date('m/d/Y:H:i:s', time())."]:Client {$ip} has tried to shutdown the server!");
				socket_write($read_sock, "Server: You are not allowed to shutdown the server!\n");
				$data = "";
				continue;
			}

			// Close all connections a leave here
			foreach ($clients as $client) {
				// Send message to client
				if ($client !== $main_sock && $client != $read_sock) {
					socket_write($client, "Server: Shutting down! Thank you for joining us.\n");
				}

				// Quit him
				socket_shutdown($client, 2);
				socket_close($client);
			} // end foreach

			// Leave the loop
			$data = "";
			$clients = array();
			continue;
		}

		// trim off the trailing/beginning white spaces
		$data = trim($data);

		// Test for HTML codes
		$tags = strip_tags($data);

		// check if there is any data after trimming off the spaces
		if (!empty($data) && $tags == $data && count($clients) > 2) {
			// Send confirmation to "chatter"
			socket_write($read_sock, "\nServer: Message accepted.\n");

			// send this to all the clients in the $clients array (except the first one, which is a listening socket)
			foreach ($clients as $send_sock) {

				// if its the listening sock or the client that we got the message from, go to the next one in the list
				if ($send_sock == $main_sock || $send_sock == $read_sock)
					continue;

				// write the message to the client -- add a newline character to the end of the message
				socket_write($send_sock, "{$ip}:{$data}\n");

			} // end of broadcast foreach
		} elseif ($tags != $data) {
			// HTML codes are not allowed
			out(__FILE__, __LINE__, '['.date('m/d/Y:H:i:s', time())."]:Client {$ip} has entered HTML code!");
			socket_write($read_sock, "Server: HTML is forbidden!\n");
		} elseif ((count($clients) == 2) && ($read_sock != $main_sock)) {
			// No one else will hear the "chatter"
			out(__FILE__, __LINE__, '['.date('m/d/Y:H:i:s', time())."]:Client {$ip} speaks with himself.");
			socket_write($read_sock, "Server: No one will hear you!\n");
		}
	} // end of reading foreach
}

// close the listening socket
socket_close($main_sock);

?>
